import "../style/home/home.css"
import { useNavigate } from "react-router-dom"
import { useEffect, useState } from "react"
import { User } from "../model/User.ts"
import { useAppFetcher } from "../App.tsx"
import { Visualizer } from "../components/Visualizer.tsx"

interface Tactic {
    id: number
    name: string
    creationDate: number
}

interface Team {
    id: number
    name: string
    picture: string
    main_color: string
    second_color: string
}

export default function HomePage() {
    type UserDataResponse = { user?: User; tactics: Tactic[]; teams: Team[] }
    const [{ tactics, teams }, setInfo] = useState<UserDataResponse>({
        tactics: [],
        teams: [],
    })

    const navigate = useNavigate()
    const fetcher = useAppFetcher()

    useEffect(() => {
        async function initUserData() {
            const response = await fetcher.fetchAPIGet("user-data")
            if (response.status == 401) {
                navigate("/login")
                return // if unauthorized
            }
            setInfo(await response.json())
        }

        initUserData()
    }, [fetcher, navigate])

    tactics!.sort((a, b) => b.creationDate - a.creationDate)

    const lastTactics = tactics.slice(0, 5)
    return <Home teams={teams!} tactics={tactics!} lastTactics={lastTactics} />
}

function Home({
    lastTactics,
    tactics,
    teams,
}: {
    lastTactics: Tactic[]
    tactics: Tactic[]
    teams: Team[]
}) {
    return (
        <div id="main">
            <Body lastTactics={lastTactics} tactics={tactics} teams={teams} />
        </div>
    )
}

function Body({
    lastTactics,
    tactics,
    teams,
}: {
    lastTactics: Tactic[]
    tactics: Tactic[]
    teams: Team[]
}) {
    const widthPersonalSpace = 78
    const widthSideMenu = 100 - widthPersonalSpace
    return (
        <div id="body">
            <PersonalSpace width={widthPersonalSpace} tactics={tactics} />
            <SideMenu
                width={widthSideMenu}
                tactics={lastTactics}
                teams={teams}
            />
        </div>
    )
}

function SideMenu({
    width,
    tactics,
    teams,
}: {
    width: number
    tactics: Tactic[]
    teams: Team[]
}) {
    return (
        <div
            id="side-menu"
            style={{
                width: width + "%",
            }}>
            <div id="side-menu-content">
                <LastTeamsSideMenu teams={teams} />
                <LastTacticsSideMenu tactics={tactics} />
            </div>
        </div>
    )
}

function PersonalSpace({
    width,
    tactics,
}: {
    width: number
    tactics: Tactic[]
}) {
    return (
        <div
            id="personal-space"
            style={{
                width: width + "%",
            }}>
            <TitlePersonalSpace />
            <BodyPersonalSpace tactics={tactics} />
        </div>
    )
}

function TitlePersonalSpace() {
    return (
        <div id="title-personal-space">
            <h2>Espace Personnel</h2>
        </div>
    )
}

function TacticGrid({ tactics }: { tactics: Tactic[] }) {
    return (
        <div id={"tactics-grid"}>
            {tactics.map((team) => (
                <TacticCard key={team.id} tactic={team} />
            ))}
        </div>
    )
}

function TacticCard({ tactic }: { tactic: Tactic }) {
    const navigate = useNavigate()
    return (
        <div className={"tactic-card"}>
            <div
                className={"tactic-card-preview"}
                onClick={() => navigate(`/tactic/${tactic.id}/edit`)}>
                <Visualizer
                    visualizerId={tactic.id.toString()}
                    tacticId={tactic.id}
                />
            </div>
            <p className="tactic-card-title">{tactic.name}</p>
        </div>
    )
}

function BodyPersonalSpace({ tactics }: { tactics: Tactic[] }) {
    return (
        <div id="body-personal-space">
            {tactics.length == 0 ? (
                <p>Aucune tactique créée !</p>
            ) : (
                <TacticGrid tactics={tactics} />
            )}
        </div>
    )
}

function LastTeamsSideMenu({ teams }: { teams: Team[] }) {
    const navigate = useNavigate()
    return (
        <div id="teams">
            <div className="titre-side-menu">
                <h2 className="title">Mes équipes</h2>
                <button className="new" onClick={() => navigate("/team/new")}>
                    +
                </button>
            </div>
            <SetButtonTeam teams={teams} />
        </div>
    )
}

function LastTacticsSideMenu({ tactics }: { tactics: Tactic[] }) {
    const navigate = useNavigate()

    return (
        <div id="tactic">
            <div className="titre-side-menu">
                <h2 className="title">Mes dernières tactiques</h2>
                <button
                    className="new"
                    id="create-tactic"
                    onClick={() => navigate("/tactic/new")}>
                    +
                </button>
            </div>
            <SetButtonTactic tactics={tactics} />
        </div>
    )
}

function SetButtonTactic({ tactics }: { tactics: Tactic[] }) {
    const lastTactics = tactics.map((tactic) => (
        <ButtonLastTactic key={tactic.id} tactic={tactic} />
    ))
    return <div className="set-button">{lastTactics}</div>
}

function SetButtonTeam({ teams }: { teams: Team[] }) {
    const listTeam = teams.map((team) => (
        <ButtonTeam key={team.id} team={team} />
    ))
    return <div className="set-button">{listTeam}</div>
}

function ButtonTeam({ team }: { team: Team }) {
    const name = truncateString(team.name, 20)
    const navigate = useNavigate()

    return (
        <div>
            <div
                id={"button-team" + team.id}
                className="button-side-menu data"
                onClick={() => {
                    navigate("/team/" + team.id)
                }}>
                {name}
            </div>
        </div>
    )
}

function ButtonLastTactic({ tactic }: { tactic: Tactic }) {
    const name = truncateString(tactic.name, 20)
    const navigate = useNavigate()

    return (
        <div
            id={"button" + tactic.id}
            className="button-side-menu data"
            onClick={() => {
                navigate("/tactic/" + tactic.id + "/edit")
            }}>
            {name}
        </div>
    )
}

function truncateString(name: string, limit: number): string {
    if (name.length > limit) {
        name = name.substring(0, limit) + "..."
    }
    return name
}
